==> linm-0.7a.tar.gz: linm-0.7a/src/ncurses/dialog_menu.h <==
/******************************************************************************
 *   Copyright (C) 2005 by la9527                                             *
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      * 
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

#ifndef __MENU_H__
#define __MENU_H__

#include "define.h"
#include "dialog.h"
#include "keycfgload.h"

namespace MLS {

///	\brief	MenuItem class
class MenuItem:public Position
{
	bool	_bEnable;
	bool	_bCur;		/// 선택 여부
	string	_sName;		///< 화면에 출력될 MenuItem 이름
	string	_sCmd;		///< 실행될 MenuItem 명령어
	string	_sCtlKey;
	
	ColorEntry	_tMenuColor;
	ColorEntry	_tMenuAColor;
	
public:
	
	///	\brief	생성자
	///	\param	n		MenuItem 이름
	///	\param	c		MenuItem 명령어
	MenuItem(	const string &n, 
				const string &c, 
				bool bEn = true,
				bool bCur = false) 
		: _sName(n), _sCmd(c), _bEnable(bEn), _bCur(bCur)
	{}
	
	// setter	===================================
	///	\brief	이름 설정함수
	///	\param	n		설정할 화면에 출력될 메뉴 Item
	void SetName(const string &n) { _sName = n;}

	///	\brief	명령 설정 함수
	///	\param	n		  설정할 명령어
	void SetCmd(const string &n) { _sCmd = n;}	
	// ============================================
	
	// getter	===================================
	///	\brief	화면에 출력될 이름 얻기 함수
	///	\return	화면에 출력될 이름
	const string &GetName() {return _sName;}

	///	\brief	명령어 얻기 함수
	///	\return	명령어
	const string &GetCmd() {return _sCmd;}
	// ============================================

	void	SetCtlKey(const string& sCtlKey)	{ _sCtlKey = sCtlKey; }
	const string &GetCtlKey()	{ return _sCtlKey; }

	///	\brief	명령 활성화 
	///	\param	bEn		Enable이면 true
	void	SetEnable(bool bEn) { _bEnable = bEn; }
	
	///	\brief	활성화 여부를 돌려준다.
	///	\return	Enable
	bool	GetEnable() { return _bEnable; }
	
	void	SetCur(bool bCur)	{ _bCur = bCur; }
	bool	GetCur()	{ return _bCur; }

	void	Color(const ColorEntry& tMenuColor, const ColorEntry& tMenuAColor)
	{
		_tMenuColor = tMenuColor;
		_tMenuAColor = tMenuAColor;
	}	

	void	Draw()
	{
		WINDOW*	pWin = _pForm->GetWin();

		if (_sName.empty())
		{
			// 빈 문자열은 줄긋기
			setcol(0, _tMenuColor.back, pWin);
			mvwhline(pWin, y, x+1, HLINE, width-2);
		}
		else
		{
		
			if (_bCur == false)
				setcol(_tMenuColor, pWin);
			else
				setcol(7, 0,  pWin);

			mvwhline(pWin, y, x+1, ' ', width-2);
			if (_bEnable == false)  setcol(8, _tMenuColor.back, pWin);
			mvwprintw(pWin, y, x+2, "%s", _sName.c_str());
			
			if (!_sCtlKey.empty())
			{
				// Ctrl, Alt 정보를 넣는다
				if (_bCur == false)
				{
					if (_bEnable == false)
						setcol(8, _tMenuColor.back, pWin);
					else
						setcol(_tMenuAColor, pWin);
				}
				else
					setcol(7, 0,  pWin);
				mvwprintw(pWin, y, x+width-2-scrstrlen(_sCtlKey), "%s", _sCtlKey.c_str());
			}
		}
		//wnoutrefresh(pWin);
	}
};

// 메뉴의 분류
///	\brief	MenuCategory class[메뉴의 분류]
class MenuCategory:public Form
{
	string _sName;		///< 메뉴 이름
	
	int _nIndex;     		///< 메뉴 Category 내에서 현재 커서 위치
	
	vector<MenuItem> 	_vItem; ///< 서브메뉴
	ColorEntry			_tMenuColor;
	ColorEntry			_tMenuAColor;

public:
	///	\brief	생성자
	///	\param	p		MenuCategory 이름
	///	\param	l		MenuCategory location
	MenuCategory(const string &p)
		: _sName(p) , _nIndex(0) 
	{
		_bNoOutRefresh = true; // Refresh 할때 doupdate 하지 않는다.
		_bNoViewUpdate = true; // 업데이트 하지 않는다.
		_bNotDrawBox = true; // 박스를 그리지 않는다.
	}

	~MenuCategory()
	{
	}

	void	MenuColor(const ColorEntry& tMenuColor, const ColorEntry& tMenuAColor)
	{
		_tMenuColor = tMenuColor;
		_tMenuAColor = tMenuAColor;
	}	

	///	\brief	이름  얻기 함수
	///	\return	이름
	const string &GetName(){return _sName;}
	
	///	\brief	MenuItem 추가 함수
	///	\param	n		MenuItem 이름
	///	\param	c		MenuItem Command
	///	\param	k		MenuItem KeyInfo
	///	\param	b		MenuItem Enable 여부 (기본 Enable)
	void AddItem(	const string &n, 
					const string &c, 
					bool b = true)
	{
		MenuItem	tMenuItem(n, c, b);
		tMenuItem.SetCtlKey(g_tKeyCfg.CmdToKey(c));
		tMenuItem.x = 0; tMenuItem.y = _vItem.size()+1;
		tMenuItem.height = 1; tMenuItem.width = width;
		_vItem.push_back(tMenuItem);
	}
	
	///	\brief	MenuItem Command 얻기함수
	///	\return	MenuItem Command
	const string &GetCmd() { return _vItem[_nIndex].GetCmd(); }

	///	\brief	MenuItem Command 얻기함수
	///	\return	MenuItem Command
	bool	GetEnable() { return _vItem[_nIndex].GetEnable(); }

	void	SetDisable(const std::string& sStr)
	{
		for(int c=0; c < _vItem.size(); c++)
		{
			if (_vItem[c].GetCmd() == sStr) _vItem[c].SetEnable(false);
		}
	}
	
	int  GetItemSize() { return _vItem.size(); }

	void SetCursor(const int nIdx)
	{
		if (nIdx < 0 || _vItem.size() <= nIdx) return;
		if (_vItem[nIdx].GetName().empty()) return;
		_nIndex = nIdx;
	}

	void DrawFirst()
	{
		werase(_pWin);
		wbkgd(_pWin, COLOR(_tMenuColor.font, _tMenuColor.back));
		
		setcol(15, _tMenuColor.back, _pWin);
		mvwaddch (_pWin, 0, 0, ULCORNER);
		mvwaddch (_pWin, 0, width-1, URCORNER);
		mvwaddch (_pWin, height-1, 0, LLCORNER);
		mvwaddch (_pWin, height-1, width-1, LRCORNER);
		mvwhline(_pWin, 0, 1, HLINE, width-2);
		mvwvline(_pWin, 1, 0, VLINE, height-2);
		mvwhline(_pWin, height-1, 1, HLINE, width-2);
		mvwvline(_pWin, 1, width-1, VLINE, height-2);
	}

	void Draw()
	{
		for (int n = 0; n < _vItem.size(); n++)
		{
			MenuItem tMenuItem = _vItem[n];
			tMenuItem.SetForm((Form*)this);
			tMenuItem.Color(_tMenuColor, _tMenuAColor);
			if (_nIndex == n)
				tMenuItem.SetCur(true);
			else
				tMenuItem.SetCur(false);
			tMenuItem.Show();
		}
	}
	
	void CursorUp()
	{
		if (_vItem.empty()) return;

		_nIndex = (_nIndex + _vItem.size() - 1) % _vItem.size();
	
		// 빈 스트링은 건너뛴다
		// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
		if (_vItem[_nIndex].GetName().empty()) CursorUp();
		if (_vItem[_nIndex].GetEnable() == false) CursorUp();
	}

	void CursorDown()
	{
		if (_vItem.empty()) return;

		_nIndex = (_nIndex + 1) % _vItem.size();
	
		// 빈 스트링은 건너뛴다
		// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
		if (_vItem[_nIndex].GetName().empty()) CursorDown();
		if (_vItem[_nIndex].GetEnable() == false) CursorDown();
	}

	bool	MouseEvent(int Y, int X, mmask_t bstate)
	{
		int nY = Y - y; int nX = X - x;
		for (int n = 0; n<_vItem.size(); n++)
		{
			if (_vItem[n].AreaChk(nY, nX))
			{
				_nIndex = n;
				return true;
			}
		}
		return false;
	}
};

///	\brief	Menu class
class Menu:public Form
{
protected:
	int _nNum;									///< 현재 선택되어 있는 item index
	bool						_bESC_Exit;		///< ESC Exit
	std::vector<MenuCategory> 	_vItem; 		///< 메뉴 뭉치들(예.윈도우에서 File 메뉴)
	std::vector<std::string>	_vCmdDisable; 	/// Disable될 Cmd list
	ColorEntry					_tColorMenu;
	ColorEntry					_tColorMenuA;
	
	bool	MouseEvent(int Y, int X, mmask_t bstate)
	{
		if (Y == 0)
		{
			for (int t = 0; t < _vItem.size(); t++)
			{
				int width = scrstrlen(_vItem[t].GetName());
				if (_vItem[t].x <= X && X <= _vItem[t].x + width)
				{
					_nNum = t;
					MenuRefresh();
					return true;
				}
			}
		}
		else
		{
			if (_vItem[_nNum].MouseEvent(Y, X, bstate) == false)
			{
				// 선택이외의 것이 존재하면 ESC와 같은 효
				_bESC_Exit = true;
				_bExit = true;
			}
			else
			{
				if (bstate & BUTTON1_DOUBLE_CLICKED)
				{
					_bExit = true;
				}
			}
		}
		return false;
	}

	// _tColorMenu, _tColorMenuA 의 컬러를 세팅한다.
	virtual void MenuColorSetting() = 0;

	// Reflash 할때 기타 화면을 Reflesh 꼭 넣어야 제대로 화면 나옴.
	virtual void MenuRefresh() {}
	
	// 각 메뉴를 넣는 가상함수
	virtual void MenuAddCatecory() = 0;

public:
	Menu() 
	{ 
		_nNum = 0; 
		//_bNoOutRefresh = true; // Refresh 할때 doupdate 하지 않는다.
		//_bNoViewUpdate = true; // 업데이트 하지 않는다.
		_bNotDrawBox = true; // 박스를 그리지 않는다.
		_bESC_Exit = false;
		x = 0; y = 0;
		height = 1; width = COLS-1;
	}
	
	///	\brief	menuCategory를 추가한다.
	///	\param	p	추가할 MenuCategory
	void AddItem(MenuCategory& p)
	{
		// . 아무런 메뉴(예. File 메뉴)가 없으면 기위치(x)를 4로한다.
		if (_vItem.empty())
			p.x = 4;
		// . 메뉴가 있다면 전 메뉴가 위치한 자리와 글자수를 참고하여 위치를 설정
		else 
		{
			p.x = _vItem.back().x + _vItem.back().GetName().size() + 2;
		}

		p.y = 1;
		p.width = 26;
		p.height = p.GetItemSize() + 2;

		p.MenuColor(_tColorMenu, _tColorMenuA);

		// . 메뉴 뭉치 추가
		_vItem.push_back(p);
	}

	string	GetCurCmd()
	{
		if (_bESC_Exit == false) { return _vItem[_nNum].GetCmd(); }
		return "";
	}
	
	///	\brief	Disable 될 명령 list 를 추가한다.
	///	\param	vCmd	disable 할 메뉴 List (command)
	void SetDefault(void)	{ _vCmdDisable.clear(); }
	void SetDisable(std::vector<std::string> vCmd) { _vCmdDisable = vCmd; }

	void	Create()
	{
		_vItem.clear();

		MenuColorSetting();
		MenuAddCatecory();
	
		// Disable 된 데이터를 바꾼다.
		for(int c=0; c<_vCmdDisable.size(); c++)
			for(int b=0; b<_vItem.size(); b++)
				_vItem[b].SetDisable(_vCmdDisable[c]);
	}
	
	void DrawFirst()
	{
		x = 0; y = 0;
		height = 1; width = COLS-1;

		// 주 메뉴를 그린다.
		// 예 "Mls  Run  File  Directory  Archive  Option" 메뉴
		setcol(_tColorMenu, _pWin);
		wmove(_pWin, y, x);
		whline(_pWin,' ', width);

		for (int t=0; t<_vItem.size(); t++)	
		{
			mvwprintw(_pWin, y, _vItem[t].x, " %s ", _vItem[t].GetName().c_str());
			g_Log.Write("MENU :: %-11s", _vItem[t].GetName().c_str());
		}
	
		// 주 메뉴에서 첫글자를 노란글자로 쓴다.(구분)
		setcol(_tColorMenuA, _pWin);
		for (int t=0; t<_vItem.size(); t++)
		{
			mvwprintw(_pWin, y, _vItem[t].x+1,
					(char*)scrstrncpy(_vItem[t].GetName(), 0, 1).c_str());
		}
	
		// 선택 된 메뉴에 검은색바탕에 글자를 씀
		setcol(7, 0, _pWin);
		mvwprintw(_pWin, y, _vItem[_nNum].x, " %s ", _vItem[_nNum].GetName().c_str());
	
		// 선택 된 메뉴에 첫번째 글자를 노란색으로 쓴다.
		setcol(_tColorMenuA.font, 0, _pWin);
		mvwprintw(_pWin, y, _vItem[_nNum].x+1,
				(char*)scrstrncpy(_vItem[_nNum].GetName(), 0, 1).c_str());

		//wnoutrefresh(_pWin);
	}

	void Draw()
	{
		if (_vItem[_nNum].GetEnable() == false) _vItem[_nNum].CursorDown();
		_vItem[_nNum].Show();
	}

	void Refresh(bool bNoOutRefresh = false)
	{
		werase(_pWin);
		_bNoOutRefresh = bNoOutRefresh;
		MenuRefresh();
		Show();
	}

	void	Execute(KeyInfo& tKeyInfo)
	{
		bool bRefesh = false;
		
		switch((int)tKeyInfo)
		{
			case KEY_DOWN:
			{
				_vItem[_nNum].CursorDown();
				break;
			}
			case KEY_UP:
			{
				_vItem[_nNum].CursorUp();
				break;
			}
			// .. 왼쪽 화살표를 누르면 왼쪽에 있는 메뉴로 이동한다.
			case KEY_LEFT:
			{
				_vItem[_nNum].Close(); // 전에 사용한 윈도우는 Close 한다.
				_nNum = (_nNum+ _vItem.size() -1) % _vItem.size();
				if (_vItem[_nNum].GetEnable() == false) _vItem[_nNum].CursorDown();
				bRefesh = true;
				break;
			}
	
			// .. 오른쪽 화살표를 누르면 오른쪽에 있는 메뉴로 이동한다.
			case KEY_RIGHT:
			{
				_vItem[_nNum].Close(); // 전에 사용한 윈도우는 Close 한다.
				_nNum = (_nNum + 1) % _vItem.size();
				if (_vItem[_nNum].GetEnable() == false) _vItem[_nNum].CursorDown();
				bRefesh = true;
				break;
			}

			case KEY_ENTER:
			{
				_bExit = true;
				break;
			}

			case KEY_ESC:
			{
				_bESC_Exit = true;
				_bExit = true;
				break;
			}
		}

		if (bRefesh) MenuRefresh();
	}
};

};

#endif

==> linm_0.7.7-1.tar.gz: linm-0.7.7/src/ncurses/dialog_menu.h <==
/******************************************************************************
 *   Copyright (C) 2005 by la9527                                             *
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      * 
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

#ifndef __MENU_H__
#define __MENU_H__

#include "define.h"
#include "dialog.h"
#include "keycfgload.h"

namespace MLS {

///	\brief	MenuItem class
class MenuItem:public Position
{
	bool	_bEnable;
	bool	_bCur;		/// 선택 여부
	string	_sName;		///< 화면에 출력될 MenuItem 이름
	string	_sCmd;		///< 실행될 MenuItem 명령어
	string	_sCtlKey;
	
	ColorEntry	_tMenuColor;
	ColorEntry	_tMenuAColor;
	
public:
	
	///	\brief	생성자
	///	\param	n		MenuItem 이름
	///	\param	c		MenuItem 명령어
	MenuItem(	const string &n, 
				const string &c, 
				bool bEn = true,
				bool bCur = false) 
		: _sName(n), _sCmd(c), _bEnable(bEn), _bCur(bCur)
	{}
	
	// setter	===================================
	///	\brief	이름 설정함수
	///	\param	n		설정할 화면에 출력될 메뉴 Item
	void SetName(const string &n) { _sName = n;}

	///	\brief	명령 설정 함수
	///	\param	n		  설정할 명령어
	void SetCmd(const string &n) { _sCmd = n;}	
	// ============================================
	
	// getter	===================================
	///	\brief	화면에 출력될 이름 얻기 함수
	///	\return	화면에 출력될 이름
	const string &GetName() {return _sName;}

	///	\brief	명령어 얻기 함수
	///	\return	명령어
	const string &GetCmd() {return _sCmd;}
	// ============================================

	void	SetCtlKey(const string& sCtlKey)	{ _sCtlKey = sCtlKey; }
	const string &GetCtlKey()	{ return _sCtlKey; }

	///	\brief	명령 활성화 
	///	\param	bEn		Enable이면 true
	void	SetEnable(bool bEn) { _bEnable = bEn; }
	
	///	\brief	활성화 여부를 돌려준다.
	///	\return	Enable
	bool	GetEnable() { return _bEnable; }
	
	void	SetCur(bool bCur)	{ _bCur = bCur; }
	bool	GetCur()	{ return _bCur; }

	void	Color(const ColorEntry& tMenuColor, const ColorEntry& tMenuAColor)
	{
		_tMenuColor = tMenuColor;
		_tMenuAColor = tMenuAColor;
	}	

	void	Draw();
};

// 메뉴의 분류
///	\brief	MenuCategory class[메뉴의 분류]
class MenuCategory:public Form
{
	string _sName;		///< 메뉴 이름
	
	int _nIndex;     		///< 메뉴 Category 내에서 현재 커서 위치
	
	vector<MenuItem> 	_vItem; ///< 서브메뉴
	ColorEntry			_tMenuColor;
	ColorEntry			_tMenuAColor;
	ViewType			_eViewType;

public:
	///	\brief	생성자
	///	\param	p		MenuCategory 이름
	///	\param	l		MenuCategory location
	MenuCategory(const string &p, const ViewType eViewType)
		: _sName(p) , _nIndex(0) 
	{
		_bNoOutRefresh = true; // Refresh 할때 doupdate 하지 않는다.
		_bNoViewUpdate = true; // 업데이트 하지 않는다.
		_bNotDrawBox = true; // 박스를 그리지 않는다.
		_eViewType = eViewType;
	}

	~MenuCategory()
	{
	}

	void	MenuColor(const ColorEntry& tMenuColor, const ColorEntry& tMenuAColor)
	{
		_tMenuColor = tMenuColor;
		_tMenuAColor = tMenuAColor;
	}	

	///	\brief	이름  얻기 함수
	///	\return	이름
	const string &GetName() const {return _sName;}
	
	///	\brief	MenuItem 추가 함수
	///	\param	n		MenuItem 이름
	///	\param	c		MenuItem Command
	///	\param	k		MenuItem KeyInfo
	///	\param	b		MenuItem Enable 여부 (기본 Enable)
	void AddItem(	const string &n, 
					const string &c, 
					bool b = true);

	///	\brief	MenuItem Command 얻기함수
	///	\return	MenuItem Command
	string 	GetCmd()  { return _vItem[_nIndex].GetCmd(); }

	///	\brief	MenuItem Command 얻기함수
	///	\return	MenuItem Command
	bool	GetEnable() { return _vItem[_nIndex].GetEnable(); }
	int  	GetItemSize() { return _vItem.size(); }

	void	SetDisable(const std::string& sStr);
	void 	SetCursor(const int nIdx);
	void	SetTypeName(const ViewType e) { _eViewType = e; }

	void 	DrawFirst();
	void 	Draw();
	
	void 	CursorUp();
	void 	CursorDown();
	bool	MouseEvent(int Y, int X, mmask_t bstate);
};

///	\brief	Menu class
class Menu:public Form
{
protected:
	int _nNum;									///< 현재 선택되어 있는 item index
	bool						_bESC_Exit;		///< ESC Exit
	std::vector<MenuCategory> 	_vItem; 		///< 메뉴 뭉치들(예.윈도우에서 File 메뉴)
	std::vector<std::string>	_vCmdDisable; 	/// Disable될 Cmd list
	ColorEntry					_tColorMenu;
	ColorEntry					_tColorMenuA;
	
	bool	MouseEvent(int Y, int X, mmask_t bstate);

	// _tColorMenu, _tColorMenuA 의 컬러를 세팅한다.
	virtual void MenuColorSetting() = 0;

	// Reflash 할때 기타 화면을 Reflesh 꼭 넣어야 제대로 화면 나옴.
	virtual void MenuRefresh() {}
	
	// 각 메뉴를 넣는 가상함수
	virtual void MenuAddCatecory() = 0;

public:
	Menu();

	void 	AddItem(MenuCategory& p);
	string	GetCurCmd();
	
	///	\brief	Disable 될 명령 list 를 추가한다.
	///	\param	vCmd	disable 할 메뉴 List (command)
	void 	SetDefault(void)	{ _vCmdDisable.clear(); }
	void 	SetDisable(std::vector<std::string> vCmd) { _vCmdDisable = vCmd; }

	void	Create();	
	void	DrawFirst();
	void 	Draw();
	void 	Refresh(bool bNoOutRefresh = false);

	void	Execute(KeyInfo& tKeyInfo);
};

};

#endif
